{% extends "oauth2_provider/base.html" %}

{% load i18n %}
{% block content %}
    <form action="" method="post">{% csrf_token %}
        <p>{% trans "Are you sure you want to delete this token?" %}</p>
        <input type="submit" value="{% trans "Delete" %}" />
    </form>
{% endblock %}
